/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: mount.cpp
 *	scope: linux client
 *	author: evgeny yakovlev
 *
 *	description:
 *	linux client 'mount' command.
 */

#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include "mount.h"
#include "misc.h"
#include "iofile.h"
#include "fs.h"
#include "elevated.h"

#include <common/exception.h>
#include <common/log.h>
#include <common/smartptr.h>

#include <alg/device.h>

using namespace bdk;

bool bcmount(int argc, char** argv, const char* fstype, bcu32_t uid, bcu32_t gid, bcu32_t mode, bool ro, const char* opts )
{
	if( argc < 3 ) {
		bc_get_log(BC_MSG_CRIT) << "Please specify container name and mount point";
		return false;
	} else if( argc > 3 ) {
		bc_get_log(BC_MSG_CRIT) << "Too many parameters";
		return false;
	}

	BCFSObject cont_path(argv[1]), mp_path(argv[2]);

	if(!cont_path.is_valid())
	{
		bc_get_log(BC_MSG_CRIT) << "Error: File " << cont_path << " does not exist";
		return false;
	}

    	if( !ro && !cont_path.access(BC_ACCESS_WRITE) )
	{
		bc_get_log(BC_MSG_CRIT) << "Error: You do not have write permissions for this container.\n"
					<< "Run with -r option to mount read-only";
        	return false;
    	}

	if(!mp_path.is_valid() || !mp_path.is_directory())
	{
		bc_get_log(BC_MSG_CRIT) << "Error: mount point " << mp_path << " does not exist";
		return false;
	}

    	if( (getuid() != 0) && (!mp_path.is_owner()) ) {
        	bc_get_log(BC_MSG_CRIT) << "Error: You are not the owner of " << mp_path;
        	return false;
	}

	// canonize both paths.
	cont_path.canonize();
	mp_path.canonize();


	//
	// load container to mount
	//


	ContainerFile cont_file(cont_path, BC_FILE_LARGE|BC_FILE_READ|((ro) ? 0 : BC_FILE_WRITE));
	BCContainerHandle cont = load_container_file(cont_file);

	if( !cont ) {
		bc_get_log(BC_MSG_CRIT) << "Could not parse container from file " << cont_path;
		return false;
	}


	// check if container is locked
	if( cont->format()->is_locked() )
	{
		bc_locked_warning(cont_path.path().c_str());
		return false;
	}


	// auth
	BCContainerViewHandle view = cont->format()->get_view(&cont_file);
	if( !view ) {
		bc_get_log(BC_MSG_CRIT) << "Error: failed authorizing container";
		return false;
	}


	// check fstype
	FSHandle fs;


	if (!fstype) {
		bc_get_log(BC_MSG_WARN) << "Warning: explicit file system type (-t option) was not provided. Mount point ownership problems can occur.";
		fs = NativeFS::get_by_id(FS_TAG_UNDEFINED);
	}
	else
	{
		fs = NativeFS::get_by_name(fstype);
		if(!fs)
		{
		        bc_get_log(BC_MSG_CRIT) << "Error: unknown file system type " << fstype;
		        return false;
		}
	}



	BCDeviceHandle dev = BCDevice::get_free();
	if( !dev ) {
		bc_get_log(BC_MSG_CRIT) << "Error: failed allocating free device";
		return false;
	}

	BCContainerAutoBinding binding = view->create_binding(&cont_file, dev, ro);

	// proceed to mount
	fs->mount(mp_path, dev, uid, gid, mode, ro, ((opts)?opts:""));

/*
	if(!ro)
	{
		cont->format()->lock();
		cont->format()->serialize(&cont_file);
	}
*/

	// set mpoint ownership
	if(!ro)
	{
		ElevatedScope sudo;

		if( 0 != chown(mp_path.path().c_str(), uid, gid) )
		{
			bc_get_log(BC_MSG_WARN) << "mount: failed changing mount point ownership: " << strerror(errno);
		}

		if( 0 != chmod(mp_path.path().c_str(), mode) )
		{
			bc_get_log(BC_MSG_WARN) << "mount: failed changing mount point mode: " << strerror(errno);
		}
	}

	binding.lock();
	bc_get_log(BC_MSG_INFO) << "Container successfuly mounted";
	return true;
}


